#include <EEPROM.h>
#include <ESP8266WiFi.h>
#include <ESP8266mDNS.h>
#include <WiFiUdp.h>
#include <ArduinoOTA.h>
#include <PubSubClient.h>
#include "RemoteDebug.h"        //https://github.com/JoaoLopesF/RemoteDebug
RemoteDebug Debug;
#include "Adafruit_MQTT.h"
#include "Adafruit_MQTT_Client.h"
WiFiClient ESP_Outlet;
PubSubClient client(ESP_Outlet);
#define HOST_NAME "ESP_Outlet"
const char* ssid = "NomeDellaTuaReteWireless";
const char* password = "PasswordDellaTuaReteWireless";
const char* mqtt_server = "192.168.1.123";   //<-- IP del tuo RaspberryPi 
const char* subscribeTopic = "HOME/Multipresa/#";
const char* inTopic1 = "HOME/Multipresa/Outlet";
const char* inTopic2 = "HOME/Multipresa/Outlet/?";
const char* outTopic = "HOME/Multipresa/Outlet/Status";

int softValue1 = HIGH;
const byte outlet = D1;
void callback(char* topic, byte* payload, unsigned int length) {
  // Conver the incoming byte array to a string
  payload[length] = '\0'; // Null terminator used to terminate the char array
  String message = (char*)payload;
  Debug.print("Message arrived [");
  Debug.print(topic);
  Debug.print("] [");
  Debug.print(message);
  Debug.println("] ");
   if (strcmp (topic, inTopic1) == 0) {
    if (message == "1")
      softValue1 = LOW;
    else if (message == "0")
      softValue1 = HIGH;
    digitalWrite(outlet, softValue1);
  }
  else if (strcmp (topic, inTopic2) == 0) {
    if (message == "?") {
      if (!softValue1)
        client.publish(outTopic, "1");
      else
        client.publish(outTopic, "0");
    }
  }
  Debug.println();
}
void reconnect() {
  //unsigned long currMillis = millis();
  // Loop until we're reconnected
  // while (!client.connected() && (currMillis - prevMillis >= Interval)) {
  while (!client.connected()) {
    Debug.println("Attempting MQTT connection...");
    // Attempt to connect
    if (client.connect("Outlet")) {
      Debug.println("Outlet connected");
      client.publish("HOME", "Hello");
      // ... and subscribe to topic
      client.subscribe(subscribeTopic);
    } else {
      Debug.print("failed, rc=");
      Debug.print(client.state());
      Debug.println(" try again in 5 second");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
void setup() {
  Serial.begin(115200);
  pinMode(outlet, OUTPUT);
  digitalWrite(outlet, softValue1);
  Debug.begin("Outlet"); // Initiaze the telnet server
  Debug.setSerialEnabled(true);
  Debug.setResetCmdEnabled(true); // Enable the reset command
  Debug.showTime(true); // To show time
  Debug.showProfiler(true); // To show profiler - time between messages of Debug
  // Good to "begin ...." and "end ...." messages
  ConnectWifi();
  OTA();
  client.setServer(mqtt_server, 1883);
  client.setCallback(callback);
}
void loop() {
  if (WiFi.waitForConnectResult() != WL_CONNECTED)
    ConnectWifi();
  ArduinoOTA.handle();
  if (!client.connected())
    reconnect();
  Debug.handle();
  client.loop();
  yield();
}
bool ConnectWifi() {
  WiFi.mode(WIFI_STA);
  WiFi.disconnect();
  delay(100);
  WiFi.begin(ssid, password);
}
void OTA() {
  ArduinoOTA.setPort(8266);
  ArduinoOTA.setHostname("ESP_Outlet");
  //ArduinoOTA.setPassword((const char *)"123");
  ArduinoOTA.onStart([]() {
    Serial.println("Start");
  });
  ArduinoOTA.onEnd([]() {
    Serial.println("\nEnd");
  });
  ArduinoOTA.onProgress([](unsigned int progress, unsigned int total) {
    Serial.printf("Progress: %u%%\r", (progress / (total / 100)));
  });
  ArduinoOTA.onError([](ota_error_t error) {
    Serial.printf("Error[%u]: ", error);
    if (error == OTA_AUTH_ERROR) Serial.println("Auth Failed");
    else if (error == OTA_BEGIN_ERROR) Serial.println("Begin Failed");
    else if (error == OTA_CONNECT_ERROR) Serial.println("Connect Failed");
    else if (error == OTA_RECEIVE_ERROR) Serial.println("Receive Failed");
    else if (error == OTA_END_ERROR) Serial.println("End Failed");
  });
  ArduinoOTA.begin();
}
